@@ -9,8 +9,7 @@ from .core import \
     Not, \
     OneOf, \
     Pattern, \
-    Remaining, \
-    Some, \
+    Many, \
     Strict, \
     String, \
     StringPattern, \
@@ -22,6 +21,7 @@ from .overload import case_distinction, Match
 from .patterns import \
     Arguments, \
     At, \
+    Attrs, \
     Between, \
     Check, \
     Contains, \
@@ -34,16 +34,27 @@ from .patterns import \
     Length, \
     Maybe, \
     NoneOf, \
-    Object, \
+    Items, \
     Regex, \
     Returns, \
+    SubclassOf, \
     Transformed
 from .try_match import Case, Default
+from .typefoo import \
+    Parameters, \
+    KwArgs, \
+    VarArgs
+
+Remaining = Many
+Some = Many
+
+Object = Items
 
 __all__ = [
     'AllOf',
     'Capture',
     'Either',
+    'Many',
     'MatchContext',
     'MatchResult',
     'Not',
@@ -68,6 +79,7 @@ __all__ = [
 
     'Arguments',
     'At',
+    'Attrs',
     'Between',
     'Check',
     'Contains',
@@ -77,11 +89,17 @@ __all__ = [
     'IsNumber',
     'IsString',
     'IsTruish',
+    'Items',
     'Length',
     'Maybe',
     'NoneOf',
     'Object',
     'Regex',
     'Returns',
+    'SubclassOf',
     'Transformed',
+
+    'Parameters',
+    'KwArgs',
+    'VarArgs',
 ]
