@@ -17,21 +17,10 @@ def get_arg_types(obj) -> List[Optional[Type]]:
     return args
 
 
-def get_arg_names(obj) -> List[str]:
-    code: CodeType = obj.__code__
-    args = []
-    argcount = code.co_argcount
-    if code.co_flags & CO_VARARGS:
-        argcount += 1
-    for ix, name in zip(range(0, argcount), code.co_varnames):
-        args.append(name)
-    return args
-
-
 def is_keyword_parameter(p: inspect.Parameter) -> bool:
-    if p.kind & inspect.Parameter.POSITIONAL_OR_KEYWORD:
+    if p.kind == inspect.Parameter.POSITIONAL_OR_KEYWORD:
         return True
-    if p.kind & inspect.Parameter.KEYWORD_ONLY:
+    if p.kind == inspect.Parameter.KEYWORD_ONLY:
         return True
     return False
 
@@ -67,30 +56,3 @@ def invoke(func, args: Union[Mapping, Iterable]):
             actual_args.append(arg)
 
     return func(*actual_args)
-
-
-def _elements(thing):
-    try:
-        yield from thing.__dict__.values()
-        return
-    except AttributeError:
-        pass
-    try:
-        yield from thing.values()
-        return
-    except AttributeError:
-        pass
-    try:
-        yield from thing
-        return
-    except TypeError:
-        pass
-    return thing
-
-
-def elements(thing):
-    for x in _elements(thing):
-        if isinstance(x, (dict, list)):
-            yield tuple(elements(x))
-        else:
-            yield x
