@@ -1,10 +1,12 @@
 from __future__ import annotations
 
 from collections import Mapping  # pylint: disable=no-name-in-module
+from copy import copy
 from dataclasses import is_dataclass
 from itertools import chain
-from typing import Optional, List, Dict, Union
-from ._util import elements
+from typing import Optional, List, Dict, Union, Tuple, Callable
+
+from .generic import AutoEqHash, AutoRepr
 
 
 class WildcardMatch:
@@ -25,14 +27,20 @@ class MatchContext:
         self.wildcards: Dict[int, WildcardMatch] = {}
         self.multimatch = multimatch
         self.strict = strict
+        self._off_the_record = []
+        self._most_recent_record = {}
+        self._match_stack = []
 
     def __setitem__(self, key, value):
+        groups = self.groups
+        if self._off_the_record:
+            groups = self._off_the_record[-1]
         if self.multimatch:
-            if key not in self.groups:
-                self.groups[key] = []
-            self.groups[key].append(value)
+            if key not in groups:
+                groups[key] = []
+            groups[key].append(value)
         else:
-            self.groups[key] = value
+            groups[key] = value
 
     def __getitem__(self, item):
         return self.groups[item]
@@ -40,57 +48,80 @@ class MatchContext:
     def __contains__(self, item):
         return item in self.groups
 
-    def match(self, value, pattern, strict=False) -> MatchResult:
-        strict = strict or self.strict
+    def keep(self):
+        for k, v in self._most_recent_record.items():
+            self[k] = v
 
-        if pattern is Ellipsis:
-            return self.matches()
+    def _pop_most_recent_record(self):
+        self._most_recent_record = self._off_the_record.pop()
 
-        if isinstance(pattern, Pattern):
-            return pattern.match(value, ctx=self, strict=strict)
+    def match(self, value, pattern, strict=False, off_the_record=False) -> MatchResult:
+        deferred: List[Callable] = []
+        self._match_stack.append((value, pattern))
+        deferred.append(lambda: self._match_stack.pop())
+        if off_the_record:
+            self._off_the_record.append({})
+            deferred.append(self._pop_most_recent_record)
 
-        if is_dataclass(value):
-            if is_dataclass(pattern):
-                pattern_type = type(pattern)
-                pattern_dict = pattern.__dict__
-            elif isinstance(pattern, Dataclass):
-                pattern_type = pattern.type
-                pattern_dict = pattern.dict
-            else:
-                return self.no_match()
+        try:
+            strict = strict or self.strict
 
-            if not issubclass(type(value), pattern_type):
-                return self.no_match()
-            return _match_mapping(value.__dict__, pattern_dict, ctx=self, strict=strict)
+            if pattern is Ellipsis:
+                return self.matches()
 
-        if isinstance(pattern, dict):
-            return _match_mapping(value, pattern, ctx=self, strict=strict)
+            if isinstance(pattern, Pattern):
+                return pattern.match(value, ctx=self, strict=strict)
 
-        if isinstance(pattern, tuple):
-            if not isinstance(value, tuple) or strict and type(value) != tuple:
-                return self.no_match()
-            return _match_sequence(value, pattern, ctx=self)
+            if is_dataclass(value):
+                if is_dataclass(pattern):
+                    pattern_type = type(pattern)
+                    pattern_dict = pattern.__dict__
+                elif isinstance(pattern, Dataclass):
+                    pattern_type = pattern.type
+                    pattern_dict = pattern.dict
+                else:
+                    return self.no_match()
 
-        if isinstance(pattern, list):
-            if strict and type(value) != list:
-                return self.no_match()
-            return _match_sequence(value, pattern, ctx=self)
+                if not issubclass(type(value), pattern_type):
+                    return self.no_match()
+                return _match_mapping(value.__dict__, pattern_dict, ctx=self, strict=strict)
 
-        if pattern == value:
-            if strict:
-                return self.match_if(type(pattern) == type(value))
-            return self.matches()
+            if type(pattern) == dict:
+                return _match_mapping(value, pattern, ctx=self, strict=strict)
 
-        return self.no_match()
+            if type(pattern) == tuple:
+                if not isinstance(value, tuple) or strict and type(value) != tuple:
+                    return self.no_match()
+                return _match_sequence(value, pattern, ctx=self)
+
+            if type(pattern) == list:
+                if strict and type(value) != list:
+                    return self.no_match()
+                return _match_sequence(value, pattern, ctx=self)
+
+            if type(pattern) == range:
+                if strict and type(value) != range:
+                    return self.no_match()
+                return _match_sequence(value, pattern, ctx=self)
+
+            if pattern == value:
+                if strict:
+                    return self.match_if(type(pattern) == type(value))
+                return self.matches()
+
+            return self.no_match()
+        finally:
+            for finalizer in deferred:
+                finalizer()
 
     def matches(self) -> MatchResult:
-        return MatchResult(matches=True, context=self)
+        return MatchResult(matches=True, context=self, match_stack=copy(self._match_stack))
 
     def no_match(self) -> MatchResult:
-        return MatchResult(matches=False, context=self)
+        return MatchResult(matches=False, context=self, match_stack=copy(self._match_stack))
 
     def match_if(self, condition: bool) -> MatchResult:
-        return MatchResult(matches=bool(condition), context=self)
+        return MatchResult(matches=bool(condition), context=self, match_stack=copy(self._match_stack))
 
     def record(self, for_pattern, value):
         id_ = id(for_pattern)
@@ -107,10 +138,10 @@ class MatchContext:
 
 
 class MatchResult(Mapping):
-    def __init__(self, *, matches: bool, context: MatchContext):
+    def __init__(self, *, matches: bool, context: MatchContext, match_stack: List[Tuple]):
         self._matches: bool = matches
         self._context: MatchContext = context
-        self._wildcard_matches: List = context.get_wildcard_matches()
+        self._match_stack: List[Tuple] = match_stack
 
     def __bool__(self):
         return self._matches
@@ -140,13 +171,27 @@ class MatchResult(Mapping):
         return self._context.groups
 
     def wildcard_matches(self) -> List:
-        return self._wildcard_matches
+        return self._context.get_wildcard_matches()
 
     def bind(self, target) -> MatchResult:
         for k, v in self.items():
             target[k] = v
         return self
 
+    def explain(self, *, short: bool = False) -> str:
+        if self._matches:
+            return "The pattern matches the given value."
+        if not self._match_stack:
+            return "Don't really know why this pattern did not match." \
+                   " Please report an issue about this at https://github.com/scravy/awesome-pattern-matching/issues"
+        reasons = []
+        match_stack: List[Tuple] = self._match_stack
+        if short:
+            match_stack = [match_stack[-1]]
+        for v, p in match_stack:
+            reasons.append(f"{v}\n...did not match the pattern:\n{repr(p)}")
+        return "\n...because:\n".join(reasons)
+
 
 class Capturable:
     def __rshift__(self, other):
@@ -194,7 +239,7 @@ def transform(pattern, f):
     return f(pattern)
 
 
-class Pattern(Capturable):
+class Pattern(Capturable, AutoEqHash, AutoRepr):
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
         raise NotImplementedError
 
@@ -210,16 +255,6 @@ class Pattern(Capturable):
     def __invert__(self):
         return Not(self)
 
-    def __eq__(self, other):
-        return type(self) == type(other) and self.__dict__ == other.__dict__
-
-    def __hash__(self):
-        return hash(tuple(elements(self)))
-
-    def __repr__(self):
-        s = tuple(f"{k}={repr(v)}" for k, v in self.__dict__.items())
-        return f"{type(self).__name__}({', '.join(s)})"
-
 
 class StringPattern:
     """Experimental"""
@@ -236,20 +271,19 @@ class String(Pattern, Nested):
 
     @staticmethod
     def match_pattern(*, remaining, pattern, ctx: MatchContext) -> Optional[str]:
-        name = ""
+        captures = []
         if isinstance(pattern, Capture):
-            name = pattern.name
-            pattern = pattern.pattern
+            captures, pattern = pattern.get_capture_pattern_chain()
         if isinstance(pattern, str):
             if remaining[:len(pattern)] == pattern:
-                if name:
-                    ctx[name] = pattern
+                for capture in captures:
+                    capture.capture(pattern, ctx=ctx)
                 return pattern
         elif isinstance(pattern, StringPattern):
             result = pattern.string_match(remaining, ctx=ctx)
             if result is not None:
-                if name:
-                    ctx[name] = result
+                for capture in captures:
+                    capture.capture(result, ctx=ctx)
                 return result
         return None
 
@@ -268,17 +302,30 @@ class String(Pattern, Nested):
 
 
 class Capture(Pattern, Nested):
-    def __init__(self, pattern, *, name):
+    def __init__(self, pattern, *, name, target=None):
         self._pattern = pattern
         self._name = name
+        self._target = target
 
     def match(self, value, *, ctx: MatchContext, strict: bool) -> MatchResult:
-        result = ctx.match(value, self._pattern)
+        result = ctx.match(value, self._pattern, strict=strict)
         if result:
-            ctx[self._name] = value
+            self.capture(value, ctx=ctx)
             return result
         return ctx.no_match()
 
+    def capture(self, value, *, ctx: MatchContext):
+        target = ctx if self._target is None else self._target
+        target[self._name] = value
+
+    def get_capture_pattern_chain(self) -> Tuple[List[Capture], Pattern]:
+        patterns = [self]
+        pattern = self._pattern
+        while isinstance(pattern, Capture):
+            patterns.append(pattern)
+            pattern = pattern.pattern
+        return patterns, pattern
+
     def descend(self, f):
         return Capture(pattern=f(self._pattern), name=self._name)
 
@@ -286,12 +333,8 @@ class Capture(Pattern, Nested):
     def pattern(self):
         return self._pattern
 
-    @property
-    def name(self):
-        return self._name
 
-
-class Some(Capturable, Nested):
+class Many(Capturable, Nested, AutoEqHash, AutoRepr):
     def __init__(self, pattern, *,
                  at_least: Optional[int] = None,
                  at_most: Optional[int] = None,
@@ -311,6 +354,9 @@ class Some(Capturable, Nested):
         self.at_least: Optional[int] = at_least
         self.at_most: Optional[int] = at_most
 
+    def __eq__(self, other):
+        return type(self) == type(other) and self.__dict__ == other.__dict__
+
     def count_ok_wrt_at_most(self, count):
         if self.at_most:
             return count <= self.at_most
@@ -322,11 +368,7 @@ class Some(Capturable, Nested):
         return True
 
     def descend(self, f):
-        return Some(pattern=f(self.pattern), at_least=self.at_least, at_most=self.at_most)
-
-
-class Remaining(Some):
-    pass
+        return Many(pattern=f(self.pattern), at_least=self.at_least, at_most=self.at_most)
 
 
 class Strict(Pattern, Nested):
@@ -445,8 +487,13 @@ def _match_mapping(value, pattern: dict, *, ctx: MatchContext, strict: bool) ->
                     keys_to_remove.append(key)
                     if key in possibly_mismatching_keys:
                         possibly_mismatching_keys.remove(key)
-                elif not isinstance(key_pattern, Underscore):
-                    possibly_mismatching_keys.add(key)
+                else:
+                    if isinstance(key_pattern, Capture):
+                        _, p = key_pattern.get_capture_pattern_chain()
+                    else:
+                        p = key_pattern
+                    if not isinstance(p, Underscore):
+                        possibly_mismatching_keys.add(key)
         for key in keys_to_remove:
             del to_be_matched[key]
     return ctx.match_if(not possibly_mismatching_keys and (not strict or not to_be_matched))
@@ -459,36 +506,34 @@ def _match_sequence(value, pattern: Union[tuple, list], *, ctx: MatchContext) ->
         return ctx.no_match()
     item_queued = False
     for current_pattern, next_pattern in zip(pattern, chain(pattern[1:], [Not(...)])):
-        name = None
-        if isinstance(current_pattern, Capture) and isinstance(current_pattern.pattern, Some):
-            name = current_pattern.name
-            current_pattern = current_pattern.pattern
-        if isinstance(current_pattern, Some):
+        captures: List[Capture] = []
+        if isinstance(current_pattern, Capture):
+            ps, p = current_pattern.get_capture_pattern_chain()
+            if isinstance(p, Many):
+                current_pattern = p
+                captures = ps
+        if isinstance(current_pattern, Many):
             count = 0
             result_value = []
             while current_pattern.count_ok_wrt_at_most(count + 1):
                 try:
-                    if item_queued:
-                        item_queued = False
-                    else:
-                        item = next(it)
+                    item = next(it)
                 except StopIteration:
                     break
-                # noinspection PyUnboundLocalVariable
-                if ctx.match(item, next_pattern):
+                if ctx.match(item, next_pattern, off_the_record=True):
                     item_queued = True
                     break
-                # noinspection PyUnboundLocalVariable
-                if not ctx.match(item, current_pattern.pattern):
+                if not ctx.match(item, current_pattern.pattern, off_the_record=True):
                     item_queued = True
                     break
-                if name:
+                ctx.keep()
+                if captures:
                     result_value.append(item)
                 count += 1
             if not current_pattern.count_ok_wrt_at_least(count):
                 return ctx.no_match()
-            if name:
-                ctx[name] = result_value
+            for capture in captures:
+                capture.capture(result_value, ctx=ctx)
             continue
         try:
             if item_queued:
@@ -497,6 +542,7 @@ def _match_sequence(value, pattern: Union[tuple, list], *, ctx: MatchContext) ->
                 item = next(it)
         except StopIteration:
             return ctx.no_match()
+        # noinspection PyUnboundLocalVariable
         result = ctx.match(item, current_pattern)
         if not result:
             return result
