@@ -1,26 +1,17 @@
 import sys
 
-from .core import MatchContext
+from .core import MatchContext, MatchResult
 
 
 class Default(BaseException):
     pass
 
 
-class TryMatch(Default):
+class TryMatch(Default, MatchResult):
     def __init__(self, value, *, ctx: MatchContext):
+        MatchResult.__init__(self, matches=False, context=ctx, match_stack=[])
         self.value = value
         self.context = ctx
-        self.result = None
-
-    def __getitem__(self, item):
-        return self.result[item]  # pylint: disable=unsubscriptable-object
-
-    def __contains__(self, item):
-        return item in self.result  # pylint: disable=unsupported-membership-test
-
-    def __iter__(self):
-        return iter(self.result)
 
 
 class NoMatch(BaseException):
@@ -36,6 +27,8 @@ def Case(pattern):
 
     result = exc.context.match(exc.value, pattern)
     if result:
-        exc.result = result
+        exc._matches = True
+        # no need to fill in match_stack as well: It is only relevant if there is no match, in which case the
+        # result object will not be accessible as we will only have it in an `except ... as result` statement
         return TryMatch
     return NoMatch
