@@ -42,9 +42,9 @@ class BasicUseCases(unittest.TestCase):
         self.assertFalse(match("", IsTruish))
 
     def test_length(self):
-        self.assertTrue(match([], Length(0)))
-        self.assertTrue(match(('abc',), Length(1)))
-        self.assertTrue(match('abc', Length(3)))
+        self.assertTrue(match([], Length(exactly=0)))
+        self.assertTrue(match(('abc',), Length(exactly=1)))
+        self.assertTrue(match('abc', Length(exactly=3)))
         self.assertTrue(match('abc', Length(at_least=1, at_most=4)))
         self.assertFalse(match('abc', Length(at_least=4, at_most=10)))
         self.assertTrue(match('abcde', Length(at_least=4, at_most=5)))
@@ -186,6 +186,7 @@ class BasicUseCases(unittest.TestCase):
         pattern = EachItem(Regex("[a-z]+"), InstanceOf(int))
         self.assertTrue(match({"a": 1, "b": 2}, pattern))
         self.assertFalse(match({"_a": 1, "b": 2}, pattern))
+        self.assertFalse(match({"a": 7.5}, pattern))
 
     def test_transformed(self):
         self.assertTrue(match("value", Transformed(len, 5)))
@@ -212,9 +213,14 @@ class BasicUseCases(unittest.TestCase):
         def f(x: int, y: bool):
             pass
 
+        # noinspection PyUnusedLocal
+        def g(*, x: int):
+            pass
+
         self.assertTrue(match(f, Arguments(x=int)))
         self.assertFalse(match(f, Strict(Arguments(x=int))))
         self.assertTrue(match(f, Strict(Arguments(x=int, y=bool))))
+        self.assertTrue(match(g, Arguments(x=int)))
 
     def test_returns(self):
         def f() -> int:
@@ -321,13 +327,13 @@ class BasicUseCases(unittest.TestCase):
         }
 
         # noinspection PyUnresolvedReferences
-        result = match(request, Object(
+        result = match(request, Items(
             api_version="v1",
-            job=Object(
+            job=Items(
                 run_at=Transformed(datetime.fromisoformat, 'time' @ _),
             ) & OneOf(
-                Object(command='command' @ InstanceOf(str)),
-                Object(spawn='container' @ InstanceOf(str)),
+                Items(command='command' @ InstanceOf(str)),
+                Items(spawn='container' @ InstanceOf(str)),
             )
         ))
         self.assertTrue(result)
@@ -335,6 +341,17 @@ class BasicUseCases(unittest.TestCase):
         self.assertEqual(result['time'], datetime(2020, 8, 27, 14, 9, 30))
         self.assertEqual(result['command'], "echo 'booya'")
 
+    def test_attrs(self):
+        @dataclass
+        class Thing:
+            foo: str
+            bar: int
+
+        self.assertTrue(match(Thing('xyz', 4711), Attrs(foo='xyz')))
+        self.assertFalse(match(Thing('xyz', 4711), Strict(Attrs(foo='xyz'))))
+        self.assertTrue(match(Thing('xyz', 4711), Attrs(foo=InstanceOf(str), bar=InstanceOf(int))))
+        self.assertTrue(match(Thing('xyz', 4711), Strict(Attrs(foo=InstanceOf(str), bar=InstanceOf(int)))))
+
     def test_transformed_exception(self):
         request = {
             "api_version": "v1",
@@ -345,8 +362,8 @@ class BasicUseCases(unittest.TestCase):
         }
 
         # noinspection PyUnresolvedReferences
-        self.assertFalse(match(request, Object(
-            job=Object(
+        self.assertFalse(match(request, Items(
+            job=Items(
                 run_at=Transformed(datetime.fromisoformat, 'time' @ _),
             )
         )))
@@ -358,13 +375,13 @@ class BasicUseCases(unittest.TestCase):
                 dct[k] = v
             return dct
 
-        result = match({'a': 1, 'b': 2, 'c': 3}, Object(a=_ >> 'x', b=_ >> 'y', c=_ >> 'z'))
+        result = match({'a': 1, 'b': 2, 'c': 3}, Items(a=_ >> 'x', b=_ >> 'y', c=_ >> 'z'))
         self.assertTrue(result)
         r = f(**result)
         self.assertEqual({'x': 1, 'y': 2, 'z': 3}, r)
 
     def test_maybe(self):
-        pattern = Object(a=...) & Maybe(Object(b=_ >> 'capture'))
+        pattern = Items(a=...) & Maybe(Items(b=_ >> 'capture'))
 
         result = match({'a': 3}, pattern)
         self.assertTrue(result)
@@ -408,7 +425,7 @@ class BasicUseCases(unittest.TestCase):
             pass
 
         # noinspection PyUnresolvedReferences
-        result = match(f, Arguments('arg_types' @ Some(...)) & Returns('return_type' @ _))
+        result = match(f, Arguments('arg_types' @ Many(...)) & Returns('return_type' @ _))
         self.assertTrue(result)
         self.assertEqual([str], result.arg_types)
         self.assertEqual(float, result.return_type)
@@ -421,3 +438,26 @@ class BasicUseCases(unittest.TestCase):
     def test_strict_match(self):
         self.assertTrue(match({'a': 3}, {'a': 3.0}))
         self.assertFalse(match({'a': 3}, {'a': 3.0}, strict=True))
+
+    def test_groups(self):
+        # noinspection PyUnresolvedReferences
+        result = match([1, 2, 3], ['a' @ _, 'b' @ _, 'c' @ _])
+        self.assertTrue(result)
+        self.assertEqual(3, len(result))
+        self.assertEqual({'a': 1, 'b': 2, 'c': 3}, result.groups())
+
+    def test_subclass_of(self):
+        result = match(bool, SubclassOf(int))
+        self.assertTrue(result)
+
+    def test_is_truish(self):
+        self.assertTrue(match([0], IsTruish))
+        self.assertTrue(match([], Not(IsTruish)))
+        self.assertFalse(match([], IsTruish))
+
+    def test_range(self):
+        self.assertTrue(match([0, 1, 2], range(0, 3)))
+        self.assertFalse(match([0, 1, 2], range(0, 3), strict=True))
+        self.assertTrue(match(range(0, 3), range(0, 3), strict=True))
+        self.assertFalse(match(range(0, 3), range(0, 2), strict=True))
+        self.assertFalse(match(range(0, 2), range(0, 3), strict=True))
