@@ -37,7 +37,7 @@ class CaseDistinctionTest(unittest.TestCase):
 
         @case_distinction
         def f(n: Match(Between(11, 20))):
-            check("It's between 1 and 10")
+            check("It's between 11 and 20")
             return 2
 
         @case_distinction
@@ -46,3 +46,23 @@ class CaseDistinctionTest(unittest.TestCase):
             return 3
 
         self.assertEqual(1, f(value))
+
+    def test_overloading(self):
+        def check(v):
+            pass
+
+        @case_distinction
+        def f(a: str, b: str):
+            return 0
+
+        @case_distinction
+        def f(a: int, b: int):
+            return 1
+
+        @case_distinction
+        def f(a: int, b: int, c: int):
+            return 2
+
+        self.assertEqual(0, f("a", "b"))
+        self.assertEqual(1, f(1, 2))
+        self.assertEqual(2, f(1, 2, 3))
