@@ -10,7 +10,7 @@ class Glob(unittest.TestCase):
     def test_empty(self):
         self.assertTrue(match(
             [1, 2, 3],
-            [1, Some(...), 2, 3]
+            [1, Many(...), 2, 3]
         ))
 
     def test_at_least(self):
@@ -18,13 +18,13 @@ class Glob(unittest.TestCase):
             [1, 2, 3],
             [
                 1,
-                Some(..., at_least=2),
+                Many(..., at_least=2),
             ]))
         self.assertFalse(match(
             [1, 2],
             [
                 1,
-                Some(..., at_least=2),
+                Many(..., at_least=2),
             ]))
 
     def test_at_most(self):
@@ -32,14 +32,14 @@ class Glob(unittest.TestCase):
             [1, 2, 3, 4],
             [
                 1,
-                Some(..., at_most=2),
+                Many(..., at_most=2),
                 4,
             ]))
         self.assertFalse(match(
             [1, 2, 3],
             [
                 1,
-                Some(..., at_most=2),
+                Many(..., at_most=2),
                 4,
             ]))
 
@@ -48,7 +48,7 @@ class Glob(unittest.TestCase):
             [1, 2, 3],
             [
                 1,
-                Some(..., at_most=2),
+                Many(..., at_most=2),
                 Remaining(...),
             ]))
 
@@ -58,9 +58,9 @@ class Glob(unittest.TestCase):
             [
                 1,
                 2,
-                Some(...),
+                Many(...),
                 6,
-                Some(...),
+                Many(...),
                 8,
                 9,
             ]
@@ -72,9 +72,9 @@ class Glob(unittest.TestCase):
             [
                 1,
                 2,
-                Capture(Some(...), name=1),
+                Capture(Many(...), name=1),
                 6,
-                Capture(Some(...), name=2),
+                Capture(Many(...), name=2),
                 8,
                 9,
             ]
@@ -82,3 +82,32 @@ class Glob(unittest.TestCase):
         self.assertTrue(result)
         self.assertEqual(result[1], [3, 4, 5])
         self.assertEqual(result[2], [7])
+
+    def test_exactly(self):
+        result = match(range(1, 10), [
+            1,
+            Many(..., exactly=3) >> 'a',
+            Many(...) >> 'b',
+            6,
+            7,
+            8,
+            9
+        ])
+        self.assertTrue(result)
+        self.assertEqual([2, 3, 4], result.a)
+        self.assertEqual([5], result.b)
+
+    def test_exactly_empty_following(self):
+        result = match(range(1, 10), [
+            1,
+            Many(..., exactly=4) >> 'a',
+            Many(...) >> 'b',
+            6,
+            7,
+            8,
+            9
+        ])
+        self.assertTrue(result)
+        self.assertEqual([2, 3, 4, 5], result.a)
+        self.assertEqual([], result.b)
+        self.assertEqual("MatchResult(matches=True, groups={'a': [2, 3, 4, 5], 'b': []})", repr(result))
