@@ -3,8 +3,6 @@ from __future__ import annotations
 import unittest
 
 # noinspection PyProtectedMember
-from dataclasses import dataclass
-
 import apm._util as util
 from apm import *
 
@@ -19,10 +17,9 @@ def g(a: int, *, b: str) -> float:
     pass
 
 
-@dataclass
-class Record:
-    foo: int
-    bar: str
+# noinspection PyUnusedLocal
+def h(a: int, *xs: float):
+    pass
 
 
 class UtilTests(unittest.TestCase):
@@ -37,6 +34,9 @@ class UtilTests(unittest.TestCase):
     def test_get_return_type(self):
         self.assertEqual(str, util.get_return_type(f))
 
+    def test_get_return_type_none(self):
+        self.assertEqual(None, util.get_return_type(lambda: None))
+
     def test_invoke(self):
         args = {
             "b": 3,
@@ -49,11 +49,6 @@ class UtilTests(unittest.TestCase):
         self.assertEqual({"a": int, "b": float, "c": str, "d": str}, util.get_kwarg_types(f))
         self.assertEqual({"a": int, "b": str}, util.get_kwarg_types(g))
 
-    def test_get_arg_names(self):
-        self.assertEqual(["a", "b", "c", "d"], util.get_arg_names(f))
-
-    def test_elements(self):
-        self.assertEqual([1, 2, 3], [x for x in util.elements([1, 2, 3])])
-        self.assertEqual([1, tuple([2]), 3], [x for x in util.elements([1, [2], 3])])
-        self.assertEqual([1, 2, 3], [x for x in util.elements({"a": 1, "b": 2, "c": 3})])
-        self.assertEqual([4, "qux"], [x for x in util.elements(Record(4, "qux"))])
+    def test_vararg_types(self):
+        self.assertEqual({"a": int}, util.get_kwarg_types(h))
+        self.assertEqual([int, float], util.get_arg_types(h))
\ No newline at end of file
